@@ -15,10 +15,14 @@
 FILENAME=$1;
 BASENAME=`basename $FILENAME`;
 origsize=`stat -c%s $FILENAME`;
-pngcrush -d /tmp/git-lint -rem alla -reduce -brute $FILENAME > /dev/null;
-newsize=`stat -c%s /tmp/git-lint/$BASENAME`;
+output=`pngcrush -d /tmp/git-lint -rem alla -reduce -brute $FILENAME 2>&1 | grep -i "not a png file"`;
+if [ "$output" != "" ]; then
+    echo 'Not a PNG file';
+    exit 1;
+fi
+newsize=`stat -c%s /tmp/git-lint/$BASENAME 2> /dev/null`;
 
-if [ $newsize -gt 0 ] && [ $newsize -lt $origsize ]; then
+if [ "$newsize" != "" ] && [ $newsize -gt 0 ] && [ $newsize -lt $origsize ]; then
     reduction=`bc <<< "scale = 2; (100*($origsize - $newsize) / $origsize)"`;
     echo "The file size can be losslessly reduced from $origsize to $newsize bytes. ($reduction % filesize reduction)";
     echo "Use: pngcrush -rem alla -reduce -brute $FILENAME <new_filename>";
